@@ -5,71 +5,109 @@
 #'
 #' @param x A vector or data frame.
 #' @param hist Should in-line histograms be returned? Default is `FALSE`.
+#' @param digits How many decimal places should the summary statistics be
+#' printed as? Default is 2.
 #'
 #' @returns
-#' `overview(x)` returns a 1-row data frame unless
-#' `x` is a data frame, in which case an object of class "overview" is returned,
-#' Under the hood this is just a a list of data frames.
+#' An object of class "overview".
+#' Under the hood this is just a list of data frames.
 #' Key summary statistics are reported in each data frame.
 #'
+#' @details
+#' No rounding of statistics is done except in printing which can be controlled
+#' either through the `digits` argument in `overview()`, or by setting the
+#' option `options(cheapr.digits)`. \cr
+#' To access the underlying data, for example the numeric summary,
+#' just use `$numeric`, e.g. `overview(rnorm(30))$numeric`.
+#'
+#' @examples
+#' library(cheapr)
+#' overview(iris)
+#'
+#' # With histograms
+#' overview(airquality, hist = TRUE)
+#'
+#' # Round to 0 decimal places
+#' overview(airquality, digits = 0)
+#'
+#' # We can set an option for all overviews
+#' options(cheapr.digits = 1)
+#' overview(rnorm(100))
+#' options(cheapr.digits = 2) # The default
 #' @rdname overview
 #' @export
-overview <- function(x, hist = FALSE){
+overview <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
  UseMethod("overview")
 }
 #' @rdname overview
 #' @export
-overview.default <- function(x, hist = FALSE){
-  out <- overview(list_as_df(list(x = x)), hist = hist)$other
+overview.default <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  overview(list_as_df(list(x = x)), hist = hist)
+}
+#' @rdname overview
+#' @export
+overview.logical <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  overview(list_as_df(list(x = as.logical(x))), hist = hist)
+}
+#' @rdname overview
+#' @export
+overview.numeric <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  out <- overview(list_as_df(list(x = as.numeric(x))), hist = hist)
+  out$cols <- NA_integer_
   out
 }
 #' @rdname overview
 #' @export
-overview.logical <- function(x, hist = FALSE){
-  out <- overview(list_as_df(list(x = as.logical(x))), hist = hist)$logical
+overview.character <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  out <- overview(list_as_df(list(x = as.character(x))), hist = hist)
+  out$cols <- NA_integer_
   out
 }
 #' @rdname overview
 #' @export
-overview.numeric <- function(x, hist = FALSE){
-  out <- overview(list_as_df(list(x = as.numeric(x))), hist = hist)$numeric
+overview.factor <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  out <- overview(list_as_df(list(x = as.factor(x))), hist = hist)
+  out$cols <- NA_integer_
   out
 }
 #' @rdname overview
 #' @export
-overview.character <- function(x, hist = FALSE){
-  out <- overview(list_as_df(list(x = as.character(x))), hist = hist)$categorical
+overview.Date <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  out <- overview(list_as_df(list(x = as.Date(x))), hist = hist)
+  out$cols <- NA_integer_
   out
 }
 #' @rdname overview
 #' @export
-overview.factor <- function(x, hist = FALSE){
-  out <- overview(list_as_df(list(x = as.factor(x))), hist = hist)$categorical
+overview.POSIXt <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  out <- overview(list_as_df(list(x = as.POSIXct(x))), hist = hist)
+  out$cols <- NA_integer_
   out
 }
 #' @rdname overview
 #' @export
-overview.Date <- function(x, hist = FALSE){
-  out <- overview(list_as_df(list(x = as.Date(x))), hist = hist)$date
+overview.ts <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
+  out <- overview(transform_all(as.data.frame(x), as.numeric), hist = hist)
+  out$time_series <- out$numeric
+  out$numeric <- sset(out$numeric, 0)
+  out$time_series$class <- class(x)[1]
   out
 }
 #' @rdname overview
 #' @export
-overview.POSIXt <- function(x, hist = FALSE){
-  out <- overview(list_as_df(list(x = as.POSIXct(x))), hist = hist)$datetime
-  out[[2]] <- utils::tail(class(x), n = 1)
-  out
-}
+overview.zoo <- overview.ts
 #' @rdname overview
 #' @export
-overview.ts <- function(x, hist = FALSE){
-  out <- overview(transform_all(as.data.frame(x), as.numeric), hist = hist)$numeric
-  out[[2]] <- utils::tail(class(x), n = 1)
-  out
-}
-#' @rdname overview
-#' @export
-overview.data.frame <- function(x, hist = FALSE){
+overview.data.frame <- function(x, hist = FALSE, digits = getOption("cheapr.digits", 2)){
+  options(cheapr.digits = digits)
   check_is_df(x)
   N <- nrow(x)
   num_cols <- ncol(x)
@@ -89,7 +127,7 @@ overview.data.frame <- function(x, hist = FALSE){
                                                             c("integer", "numeric")), FALSE)]
   date_vars <- data_nms[vapply(skim_df, function(x) inherits(x, "Date"), FALSE)]
   datetime_vars <- data_nms[vapply(skim_df, function(x) inherits(x, "POSIXt"),  FALSE)]
-  ts_vars <- data_nms[vapply(skim_df, function(x) inherits(x, "ts"),  FALSE)]
+  ts_vars <- data_nms[vapply(skim_df, function(x) inherits(x, c("ts", "zoo")),  FALSE)]
   cat_vars <- data_nms[vapply(skim_df, is.factor, FALSE)]
   other_vars <- setdiff_(data_nms, c(lgl_vars, num_vars, date_vars,
                                      datetime_vars, ts_vars, cat_vars))
@@ -98,7 +136,7 @@ overview.data.frame <- function(x, hist = FALSE){
 
   lgl_data <- df_select(skim_df, lgl_vars)
   which_lgl <- which_in(out[["col"]], lgl_vars)
-  lgl_out <- out[which_lgl, , drop = FALSE]
+  lgl_out <- sset(out, which_lgl)
   value_size <- min(length(which_lgl), 1L)
   lgl_out <- df_add_cols(lgl_out, list(n_missing = NA_integer_[value_size]))
   lgl_out <- df_add_cols(lgl_out, list(p_complete = NA_real_[value_size]))
@@ -117,7 +155,7 @@ overview.data.frame <- function(x, hist = FALSE){
 
   num_data <- df_select(skim_df, num_vars)
   which_num <- which_in(out[["col"]], num_vars)
-  num_out <- out[which_num, , drop = FALSE]
+  num_out <- sset(out, which_num)
   value_size <- min(length(which_num), 1L)
   num_out <- df_add_cols(num_out, list(n_missing = NA_integer_[value_size]))
   num_out <- df_add_cols(num_out, list(p_complete = NA_real_[value_size]))
@@ -159,7 +197,7 @@ overview.data.frame <- function(x, hist = FALSE){
 
   date_data <- df_select(skim_df, date_vars)
   which_date <- which_in(out[["col"]], date_vars)
-  date_out <- out[which_date, , drop = FALSE]
+  date_out <- sset(out, which_date)
   value_size <- min(length(which_date), 1L)
   date_out <- df_add_cols(date_out, list(n_missing = NA_integer_[value_size]))
   date_out <- df_add_cols(date_out, list(p_complete = NA_real_[value_size]))
@@ -183,7 +221,7 @@ overview.data.frame <- function(x, hist = FALSE){
   datetime_data <- df_select(skim_df, datetime_vars)
   datetime_data <- transform_all(datetime_data, as.POSIXct)
   which_datetime <- which_in(out[["col"]], datetime_vars)
-  datetime_out <- out[which_datetime, , drop = FALSE]
+  datetime_out <- sset(out, which_datetime)
   value_size <- min(length(which_datetime), 1L)
   datetime_out <- df_add_cols(datetime_out, list(n_missing = NA_integer_[value_size]))
   datetime_out <- df_add_cols(datetime_out, list(p_complete = NA_real_[value_size]))
@@ -201,7 +239,7 @@ overview.data.frame <- function(x, hist = FALSE){
     datetime_out$min <- .POSIXct(datetime_out$min, tz = "UTC")
     datetime_out$max <- .POSIXct(datetime_out$max, tz = "UTC")
     for (i in seq_len(nrow(datetime_out))){
-      datetime_out[["tzone"]][i] <- tzone(x[[datetime_out[["col"]][i]]])
+      datetime_out[["tzone"]][i] <- tzone(skim_df[[datetime_out[["col"]][i]]])
     }
   }
 
@@ -209,14 +247,16 @@ overview.data.frame <- function(x, hist = FALSE){
 
   ts_data <- df_select(skim_df, ts_vars)
   which_ts <- which_in(out[["col"]], ts_vars)
-  ts_out <- out[which_ts, , drop = FALSE]
+  ts_out <- sset(out, which_ts)
   if (N > 0L && length(which_ts) > 0) {
     ts_overviews <- new_list(nrow(ts_out))
     for (i in seq_along(ts_overviews)){
-      ts_overviews[[i]] <- overview(ts_data[[ts_out[["col"]][i]]], hist = hist)
+      ts_overviews[[i]] <- overview(ts_data[[ts_out[["col"]][i]]], hist = hist)$time_series
       if (length(attr(ts_overviews[[i]], "row.names")) > 1){
         ts_overviews[[i]][["col"]] <- paste0(ts_out[["col"]][i], "_",
                                              ts_overviews[[i]][["col"]])
+      } else {
+        ts_overviews[[i]][["col"]] <- ts_vars[i]
       }
     }
     ts_out <- collapse::rowbind(ts_overviews)
@@ -226,7 +266,7 @@ overview.data.frame <- function(x, hist = FALSE){
 
   cat_data <- df_select(skim_df, cat_vars)
   which_cat <- which_in(out[["col"]], cat_vars)
-  cat_out <- out[which_cat, , drop = FALSE]
+  cat_out <- sset(out, which_cat)
   value_size <- min(length(which_cat), 1L)
   cat_out <- df_add_cols(cat_out, list(n_missing = NA_integer_[value_size]))
   cat_out <- df_add_cols(cat_out, list(p_complete = NA_real_[value_size]))
@@ -245,7 +285,7 @@ overview.data.frame <- function(x, hist = FALSE){
     cat_out$max <- as.character(cat_out$max)
     for (i in seq_len(nrow(cat_out))){
       if (cat_out[["class"]][i] == "factor"){
-        cat_out[["n_levels"]][i] <- length(levels(x[[cat_out[["col"]][i]]]))
+        cat_out[["n_levels"]][i] <- length(levels(skim_df[[cat_out[["col"]][i]]]))
       }
     }
   }
@@ -254,7 +294,7 @@ overview.data.frame <- function(x, hist = FALSE){
 
   other_data <- df_select(skim_df, other_vars)
   which_other <- which_in(out[["col"]], other_vars)
-  other_out <- out[which_other, , drop = FALSE]
+  other_out <- sset(out, which_other)
   value_size <- min(length(which_other), 1L)
   other_out <- df_add_cols(other_out, list(n_missing = NA_integer_[value_size]))
   other_out <- df_add_cols(other_out, list(p_complete = NA_real_[value_size]))
@@ -273,7 +313,7 @@ overview.data.frame <- function(x, hist = FALSE){
   }
 
   out <- list(
-    nrow = N, ncol = num_cols,
+    obs = N, cols = num_cols,
     logical = lgl_out,
     numeric = num_out,
     date = date_out,
@@ -286,14 +326,14 @@ overview.data.frame <- function(x, hist = FALSE){
   out
 }
 #' @export
-print.overview <- function(x, max = NULL, ...){
+print.overview <- function(x, max = NULL, digits = getOption("cheapr.digits", 2), ...){
   # max_rows <- getOption("tibble.print_max", 20)
   # max_cols <- getOption("tibble.width", NULL)
   # max_extra_cols <- getOption("tibble.max_extra_cols", 100)
   # options(tibble.print_max = 10)
   # options(tibble.width = 100)
   # options(tibble.max_extra_cols = 10)
-  cat(paste("rows:", x$nrow, "cols:", x$ncol), "\n")
+  cat(paste("obs:", x$obs, "\ncols:", x$cols), "\n")
   # for (data_type in names(x)[-(1:2)]){
   #   if (nrow(x[[data_type]])){
   #     cat(paste("\n-----", data_type, "-----\n"))
@@ -301,30 +341,30 @@ print.overview <- function(x, max = NULL, ...){
   #   }
   # }
   if (nrow(x$logical)){
-    x$logical$p_complete <- pretty_num(round(x$logical$p_complete, 2))
+    x$logical$p_complete <- pretty_num(round(x$logical$p_complete, digits))
     cat("\n----- Logical -----\n")
     print(x$logical)
   }
   if (nrow(x$numeric)){
-    x$numeric$p_complete <- pretty_num(round(x$numeric$p_complete, 2))
-    x$numeric$mean <- pretty_num(round(x$numeric$mean, 2))
-    x$numeric$p0 <- pretty_num(round(x$numeric$p0, 2))
-    x$numeric$p25 <- pretty_num(round(x$numeric$p25, 2))
-    x$numeric$p50 <- pretty_num(round(x$numeric$p50, 2))
-    x$numeric$p75 <- pretty_num(round(x$numeric$p75, 2))
-    x$numeric$p100 <- pretty_num(round(x$numeric$p100, 2))
-    x$numeric$iqr <- pretty_num(round(x$numeric$iqr, 2))
-    x$numeric$sd <- pretty_num(round(x$numeric$sd, 2))
+    x$numeric$p_complete <- pretty_num(round(x$numeric$p_complete, digits))
+    x$numeric$mean <- pretty_num(round(x$numeric$mean, digits))
+    x$numeric$p0 <- pretty_num(round(x$numeric$p0, digits))
+    x$numeric$p25 <- pretty_num(round(x$numeric$p25, digits))
+    x$numeric$p50 <- pretty_num(round(x$numeric$p50, digits))
+    x$numeric$p75 <- pretty_num(round(x$numeric$p75, digits))
+    x$numeric$p100 <- pretty_num(round(x$numeric$p100, digits))
+    x$numeric$iqr <- pretty_num(round(x$numeric$iqr, digits))
+    x$numeric$sd <- pretty_num(round(x$numeric$sd, digits))
     cat("\n----- Numeric -----\n")
     print(x$numeric)
   }
   if (nrow(x$date)){
-    x$date$p_complete <- pretty_num(round(x$date$p_complete, 2))
+    x$date$p_complete <- pretty_num(round(x$date$p_complete, digits))
     cat("\n----- Dates -----\n")
     print(x$date)
   }
   if (nrow(x$datetime)){
-    x$datetime$p_complete <- pretty_num(round(x$datetime$p_complete, 2))
+    x$datetime$p_complete <- pretty_num(round(x$datetime$p_complete, digits))
     # An overview list contains a 'min' & 'max' variable of date-times
     # This is UTC because R can't handle a date-time with multiple time-zones
     # And so we want to print it in local-time
@@ -339,29 +379,29 @@ print.overview <- function(x, max = NULL, ...){
     }
     x$datetime$min <- datetime_chr_min
     x$datetime$max <- datetime_chr_max
-    cat("\n----- Date-times -----\n")
+    cat("\n----- Date-Times -----\n")
     print(x$datetime)
   }
   if (nrow(x$time_series)){
-    x$time_series$p_complete <- pretty_num(round(x$time_series$p_complete, 2))
-    x$time_series$mean <- pretty_num(round(x$time_series$mean, 2))
-    x$time_series$p0 <- pretty_num(round(x$time_series$p0, 2))
-    x$time_series$p25 <- pretty_num(round(x$time_series$p25, 2))
-    x$time_series$p50 <- pretty_num(round(x$time_series$p50, 2))
-    x$time_series$p75 <- pretty_num(round(x$time_series$p75, 2))
-    x$time_series$p100 <- pretty_num(round(x$time_series$p100, 2))
-    x$time_series$iqr <- pretty_num(round(x$time_series$iqr, 2))
-    x$time_series$sd <- pretty_num(round(x$time_series$sd, 2))
+    x$time_series$p_complete <- pretty_num(round(x$time_series$p_complete, digits))
+    x$time_series$mean <- pretty_num(round(x$time_series$mean, digits))
+    x$time_series$p0 <- pretty_num(round(x$time_series$p0, digits))
+    x$time_series$p25 <- pretty_num(round(x$time_series$p25, digits))
+    x$time_series$p50 <- pretty_num(round(x$time_series$p50, digits))
+    x$time_series$p75 <- pretty_num(round(x$time_series$p75, digits))
+    x$time_series$p100 <- pretty_num(round(x$time_series$p100, digits))
+    x$time_series$iqr <- pretty_num(round(x$time_series$iqr, digits))
+    x$time_series$sd <- pretty_num(round(x$time_series$sd, digits))
     cat("\n----- Time-Series -----\n")
     print(x$time_series)
   }
   if (nrow(x$categorical)){
-    x$categorical$p_complete <- pretty_num(round(x$categorical$p_complete, 2))
+    x$categorical$p_complete <- pretty_num(round(x$categorical$p_complete, digits))
     cat("\n----- Categorical -----\n")
     print(x$categorical)
   }
   if (nrow(x$other)){
-    x$other$p_complete <- pretty_num(round(x$other$p_complete, 2))
+    x$other$p_complete <- pretty_num(round(x$other$p_complete, digits))
     cat("\n----- Other -----\n")
     print(x$other)
   }
@@ -394,7 +434,7 @@ transform_all <- function(data, .fn){
   data
 }
 summarise_all <- function(data, .fn, size = 1){
-  out <- data[seq_len(size), , drop = FALSE]
+  out <- sset(data, seq_len(size))
   attr(out, "row.names") <- .set_row_names(size)
   for (col in names(out)){
     out[[col]] <- .fn(data[[col]])
